
#region copyright notice
//Copyright (c) 2005 Dariush Tasdighi (http://www.IranianExperts.com)
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:
//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.
#endregion

namespace IranianExperts.RSS
{
	public class RSSChannel
	{
		private string _docs = "";
		private string _link = "";
		private string _title = "";
		private string _pubDate = "";
		private string _generator = "";
		private string _webMaster = "";
		private string _copyright = "";
		private string _description = "";
		private string _lastBuildDate = "";
		private string _managingEditor = "";

		private RSSLanguage _language = RSSLanguage.en_US;

		/// <summary>
		/// [Optional] - A URL that points to the documentation for the format used in the RSS file. It's probably a pointer to this page. It's for people who might stumble across an RSS file on a Web server 25 years from now and wonder what it is.
		/// </summary>
		public string Docs
		{
			get
			{
				return(_docs);
			}
			set
			{
				_docs = value;
			}
		}

		/// <summary>
		/// [Required] - The URL to the HTML website corresponding to the channel.
		/// </summary>
		public string Link
		{
			get
			{
				return(_link);
			}
			set
			{
				_link = value;
			}
		}

		/// <summary>
		/// [Required] - The name of the channel. It's how people refer to your service. If you have an HTML website that contains the same information as your RSS file, the title of your channel should be the same as the title of your website.
		/// </summary>
		public string Title
		{
			get
			{
				return(_title);
			}
			set
			{
				_title = value;
			}
		}

		/// <summary>
		/// [Optional] - The publication date for the content in the channel. All date-times in RSS conform to the Date and Time Specification of RFC 822, with the exception that the year may be expressed with two characters or four characters (four preferred). RFC 822 Formated date.
		/// </summary>
		public string PubDate
		{
			get
			{
				return(_pubDate);
			}
			set
			{
				_pubDate = value;
			}
		}
		/// <summary>
		/// [Optional] - A string indicating the program used to generate the channel.
		/// </summary>
		public string Generator
		{
			get
			{
				return(_generator);
			}
			set
			{
				_generator = value;
			}
		}

		/// <summary>
		/// [Optional] - Email address for person responsible for technical issues relating to channel.
		/// </summary>
		public string WebMaster
		{
			get
			{
				return(_webMaster);
			}
			set
			{
				_webMaster = value;
			}
		}

		/// <summary>
		/// [Optional] - Copyright notice for content in the channel.
		/// </summary>
		public string Copyright
		{
			get
			{
				return(_copyright);
			}
			set
			{
				_copyright = value;
			}
		}

		/// <summary>
		/// [Required] - Phrase or sentence describing the channel.
		/// </summary>
		public string Description
		{
			get
			{
				return(_description);
			}
			set
			{
				_description = value;
			}
		}

		/// <summary>
		/// [Optional] - The last time the content of the channel changed. RFC 822 Formated date.
		/// </summary>
		public string LastBuildDate
		{
			get
			{
				return(_lastBuildDate);
			}
			set
			{
				_lastBuildDate = value;
			}
		}

		/// <summary>
		/// [Optional] - Email address for person responsible for editorial content.
		/// </summary>
		public string ManagingEditor
		{
			get
			{
				return(_managingEditor);
			}
			set
			{
				_managingEditor = value;
			}
		}

		/// <summary>
		/// [Optional] - The language the channel is written in. This allows aggregators to group all Italian language sites, for example, on a single page. A list of allowable values for this element, as provided by Netscape, is here. You may also use values defined by the W3C.
		/// </summary>
		public RSSLanguage Language
		{
			get
			{
				return(_language);
			}
			set
			{
				_language = value;
			}
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="title"></param>
		/// <param name="link"></param>
		/// <param name="description"></param>
		public RSSChannel(string title, string link, string description)
		{
			_link = link;
			_title = title;
			_description = description;
		}

		#region Support
		public static string Owner
		{
			get
			{
				return("Dariush Tasdighi - From Tehran, Iran");
			}
		}

		public static string Version
		{
			get
			{
				return("1.0.0");
			}
		}

		public static string Support
		{
			get
			{
				return("Dariush@IranianExperts.com;DariushT@GMail.com;DariushTasdighi@Yahoo.com");
			}
		}

		public static string Homepage
		{
			get
			{
				return("http://www.IranianExperts.com;http://groups.yahoo.com/group/IranianExperts");
			}
		}

		public static string UpdatedDate
		{
			get
			{
				return("14 Feb 2005");
			}
		}
		#endregion
	}
}
