
#region copyright notice
//Copyright (c) 2005 Dariush Tasdighi (http://www.IranianExperts.com)
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:
//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.
#endregion


namespace IranianExperts.RSS
{
	public class RSSItem
	{
		private string _link = "";
		private string _title = "";
		private string _author = "";
		private string _pubDate = "";
		private string _comment = "";
		private string _description = "";

		/// <summary>
		/// [Optional] - The URL of the item.
		/// </summary>
		public string Link
		{
			get
			{
				return(_link);
			}
			set
			{
				_link = value;
			}
		}

		/// <summary>
		/// [Optional] - The title of the item.
		/// </summary>
		public string Title
		{
			get
			{
				return(_title);
			}
			set
			{
				_title = value;
			}
		}

		/// <summary>
		/// [Optional] - Email address of the author of the item.
		/// </summary>
		public string Author
		{
			get
			{
				return(_author);
			}
			set
			{
				_author = value;
			}
		}

		/// <summary>
		/// [Optional] - Indicates when the item was published. RFC 822 Formated date.
		/// </summary>
		public string PubDate
		{
			get
			{
				return(_pubDate);
			}
			set
			{
				_pubDate = value;
			}
		}

		/// <summary>
		/// [Optional] - URL of a page for comments relating to the item.
		/// </summary>
		public string Comment
		{
			get
			{
				return(_comment);
			}
			set
			{
				_comment = value;
			}
		}

		/// <summary>
		/// [Optional] - The item synopsis.
		/// </summary>
		public string Description
		{
			get
			{
				return(_description);
			}
			set
			{
				_description = value;
			}
		}

		private string _guid = "";

		public string Guid
		{
			get { return _guid; }
			set { _guid = value; }
		}


		/// <summary>
		/// Default Constructor
		/// </summary>
		public RSSItem()
		{
		}

		/// <summary>
		/// Constructor
		/// </summary>
		public RSSItem(string title)
		{
			_title = title;
		}

		/// <summary>
		/// Constructor
		/// </summary>
		public RSSItem(string title, string link)
		{
			_link = link;
			_title = title;
		}

		#region Support
		public static string Owner
		{
			get
			{
				return("Dariush Tasdighi - From Tehran, Iran");
			}
		}

		public static string Version
		{
			get
			{
				return("1.0.0");
			}
		}

		public static string Support
		{
			get
			{
				return("Dariush@IranianExperts.com;DariushT@GMail.com;DariushTasdighi@Yahoo.com");
			}
		}

		public static string Homepage
		{
			get
			{
				return("http://www.IranianExperts.com;http://groups.yahoo.com/group/IranianExperts");
			}
		}

		public static string UpdatedDate
		{
			get
			{
				return("14 Feb 2005");
			}
		}
		#endregion
	}
}
