
#region copyright notice
//Copyright (c) 2005 Dariush Tasdighi (http://www.IranianExperts.com)
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:
//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.
#endregion


namespace IranianExperts.RSS
{
	public class RSSRoot
	{
		private System.IO.Stream _outputStream;

		private RSSImage _image;
		private RSSChannel _channel;
		private RSSItemCollection _items = new RSSItemCollection();

		/// <summary>
		/// The XML context will be written in this stream.
		/// </summary>
		public System.IO.Stream OutputStream
		{
			get
			{
				return(_outputStream);
			}
			set
			{
				_outputStream = value;
			}
		}

		/// <summary>
		/// Image Tag.
		/// </summary>
		public RSSImage Image
		{
			get
			{
				return(_image);
			}
		}

		/// <summary>
		/// Channel Tag.
		/// </summary>
		public RSSChannel Channel
		{
			get
			{
				return(_channel);
			}
		}

		/// <summary>
		/// Items Tag.
		/// </summary>
		public RSSItemCollection Items
		{
			get
			{
				return(_items);
			}
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="channel"></param>
		/// <param name="outputStream"></param>
		public RSSRoot(RSSChannel channel, System.IO.Stream outputStream)
		{
			_channel = channel;
			_outputStream = outputStream;
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="channel"></param>
		/// <param name="image"></param>
		/// <param name="outputStream"></param>
		public RSSRoot(RSSChannel channel, RSSImage image, System.IO.Stream outputStream)
		{
			_image = image;
			_channel = channel;
			_outputStream = outputStream;
		}

		#region Support
		public static string Owner
		{
			get
			{
				return("Dariush Tasdighi - From Tehran, Iran");
			}
		}

		public static string Version
		{
			get
			{
				return("1.0.0");
			}
		}

		public static string Support
		{
			get
			{
				return("Dariush@IranianExperts.com;DariushT@GMail.com;DariushTasdighi@Yahoo.com");
			}
		}

		public static string Homepage
		{
			get
			{
				return("http://www.IranianExperts.com;http://groups.yahoo.com/group/IranianExperts");
			}
		}

		public static string UpdatedDate
		{
			get
			{
				return("14 Feb 2005");
			}
		}
		#endregion
	}
}
